@@ -13,7 +13,6 @@
  *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
- *
  */
 package org.apache.bcel.util;
 
@@ -37,9 +36,13 @@ import java.util.Locale;
 import java.util.Objects;
 import java.util.StringTokenizer;
 import java.util.Vector;
+import java.util.stream.Collectors;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipFile;
 
+import org.apache.bcel.classfile.JavaClass;
+import org.apache.bcel.classfile.Utility;
+
 /**
  * Loads class files from the CLASSPATH. Inspired by sun.tools.ClassPath.
  */
@@ -143,7 +146,7 @@ public class ClassPath implements Closeable {
 
         /**
          * @return base path of found class, i.e. class is contained relative to that path, which may either denote a directory,
-         *         or zip file
+         *         or ZIP file
          */
         String getBase();
 
@@ -169,7 +172,7 @@ public class ClassPath implements Closeable {
         long getTime();
     }
 
-    private static class Dir extends AbstractPathEntry {
+    private static final class Dir extends AbstractPathEntry {
 
         private final String dir;
 
@@ -251,7 +254,7 @@ public class ClassPath implements Closeable {
         }
     }
 
-    private static class Jar extends AbstractZip {
+    private static final class Jar extends AbstractZip {
 
         Jar(final ZipFile zip) {
             super(zip);
@@ -259,12 +262,12 @@ public class ClassPath implements Closeable {
 
         @Override
         protected String toEntryName(final String name, final String suffix) {
-            return packageToFolder(name) + suffix;
+            return Utility.packageToPath(name) + suffix;
         }
 
     }
 
-    private static class JrtModule extends AbstractPathEntry {
+    private static final class JrtModule extends AbstractPathEntry {
 
         private final Path modulePath;
 
@@ -280,7 +283,7 @@ public class ClassPath implements Closeable {
 
         @Override
         ClassFile getClassFile(final String name, final String suffix) {
-            final Path resolved = modulePath.resolve(packageToFolder(name) + suffix);
+            final Path resolved = modulePath.resolve(Utility.packageToPath(name) + suffix);
             if (Files.exists(resolved)) {
                 return new ClassFile() {
 
@@ -347,7 +350,7 @@ public class ClassPath implements Closeable {
 
     }
 
-    private static class JrtModules extends AbstractPathEntry {
+    private static final class JrtModules extends AbstractPathEntry {
 
         private final ModularRuntimeImage modularRuntimeImage;
         private final JrtModule[] modules;
@@ -413,7 +416,7 @@ public class ClassPath implements Closeable {
 
     }
 
-    private static class Module extends AbstractZip {
+    private static final class Module extends AbstractZip {
 
         Module(final ZipFile zip) {
             super(zip);
@@ -421,7 +424,7 @@ public class ClassPath implements Closeable {
 
         @Override
         protected String toEntryName(final String name, final String suffix) {
-            return "classes/" + packageToFolder(name) + suffix;
+            return "classes/" + Utility.packageToPath(name) + suffix;
         }
 
     }
@@ -433,7 +436,7 @@ public class ClassPath implements Closeable {
 
     private static final FilenameFilter MODULES_FILTER = (dir, name) -> {
         name = name.toLowerCase(Locale.ENGLISH);
-        return name.endsWith(".jmod");
+        return name.endsWith(org.apache.bcel.classfile.Module.EXTENSION);
     };
 
     public static final ClassPath SYSTEM_CLASS_PATH = new ClassPath(getClassPath());
@@ -486,23 +489,16 @@ public class ClassPath implements Closeable {
         final List<String> dirs = new ArrayList<>();
         getPathComponents(extDirs, dirs);
         for (final String d : dirs) {
-            final File ext_dir = new File(d);
-            final String[] extensions = ext_dir.list(ARCHIVE_FILTER);
+            final File extDir = new File(d);
+            final String[] extensions = extDir.list(ARCHIVE_FILTER);
             if (extensions != null) {
                 for (final String extension : extensions) {
-                    list.add(ext_dir.getPath() + File.separatorChar + extension);
+                    list.add(extDir.getPath() + File.separatorChar + extension);
                 }
             }
         }
 
-        final StringBuilder buf = new StringBuilder();
-        String separator = "";
-        for (final String path : list) {
-            buf.append(separator);
-            separator = File.pathSeparator;
-            buf.append(path);
-        }
-        return buf.toString().intern();
+        return list.stream().collect(Collectors.joining(File.pathSeparator));
     }
 
     private static void getPathComponents(final String path, final List<String> list) {
@@ -518,15 +514,11 @@ public class ClassPath implements Closeable {
         }
     }
 
-    static String packageToFolder(final String name) {
-        return name.replace('.', '/');
-    }
+    private final String classPathString;
 
-    private final String classPath;
+    private final ClassPath parent;
 
-    private ClassPath parent;
-
-    private final AbstractPathEntry[] paths;
+    private final List<AbstractPathEntry> paths;
 
     /**
      * Search for classes in CLASSPATH.
@@ -538,34 +530,25 @@ public class ClassPath implements Closeable {
         this(getClassPath());
     }
 
-    public ClassPath(final ClassPath parent, final String classPath) {
-        this(classPath);
-        this.parent = parent;
-    }
-
-    /**
-     * Search for classes in given path.
-     *
-     * @param classPath
-     */
     @SuppressWarnings("resource")
-    public ClassPath(final String classPath) {
-        this.classPath = classPath;
-        final List<AbstractPathEntry> list = new ArrayList<>();
-        for (final StringTokenizer tokenizer = new StringTokenizer(classPath, File.pathSeparator); tokenizer.hasMoreTokens();) {
+    public ClassPath(final ClassPath parent, final String classPathString) {
+        this.parent = parent;
+        this.classPathString = Objects.requireNonNull(classPathString, "classPathString");
+        this.paths = new ArrayList<>();
+        for (final StringTokenizer tokenizer = new StringTokenizer(classPathString, File.pathSeparator); tokenizer.hasMoreTokens();) {
             final String path = tokenizer.nextToken();
             if (!path.isEmpty()) {
                 final File file = new File(path);
                 try {
                     if (file.exists()) {
                         if (file.isDirectory()) {
-                            list.add(new Dir(path));
-                        } else if (path.endsWith(".jmod")) {
-                            list.add(new Module(new ZipFile(file)));
+                            paths.add(new Dir(path));
+                        } else if (path.endsWith(org.apache.bcel.classfile.Module.EXTENSION)) {
+                            paths.add(new Module(new ZipFile(file)));
                         } else if (path.endsWith(ModularRuntimeImage.MODULES_PATH)) {
-                            list.add(new JrtModules(ModularRuntimeImage.MODULES_PATH));
+                            paths.add(new JrtModules(ModularRuntimeImage.MODULES_PATH));
                         } else {
-                            list.add(new Jar(new ZipFile(file)));
+                            paths.add(new Jar(new ZipFile(file)));
                         }
                     }
                 } catch (final IOException e) {
@@ -575,27 +558,37 @@ public class ClassPath implements Closeable {
                 }
             }
         }
-        paths = new AbstractPathEntry[list.size()];
-        list.toArray(paths);
+    }
+
+    /**
+     * Search for classes in given path.
+     *
+     * @param classPath
+     */
+    public ClassPath(final String classPath) {
+        this(null, classPath);
     }
 
     @Override
     public void close() throws IOException {
-        if (paths != null) {
-            for (final AbstractPathEntry path : paths) {
-                path.close();
-            }
+        for (final AbstractPathEntry path : paths) {
+            path.close();
         }
-
     }
 
     @Override
-    public boolean equals(final Object o) {
-        if (o instanceof ClassPath) {
-            final ClassPath cp = (ClassPath) o;
-            return classPath.equals(cp.toString());
+    public boolean equals(final Object obj) {
+        if (this == obj) {
+            return true;
         }
-        return false;
+        if (obj == null) {
+            return false;
+        }
+        if (getClass() != obj.getClass()) {
+            return false;
+        }
+        final ClassPath other = (ClassPath) obj;
+        return Objects.equals(classPathString, other.classPathString);
     }
 
     /**
@@ -604,7 +597,7 @@ public class ClassPath implements Closeable {
      * @throws IOException if an I/O error occurs.
      */
     public byte[] getBytes(final String name) throws IOException {
-        return getBytes(name, ".class");
+        return getBytes(name, JavaClass.EXTENSION);
     }
 
     /**
@@ -636,13 +629,13 @@ public class ClassPath implements Closeable {
      * @throws IOException if an I/O error occurs.
      */
     public ClassFile getClassFile(final String name) throws IOException {
-        return getClassFile(name, ".class");
+        return getClassFile(name, JavaClass.EXTENSION);
     }
 
     /**
      * @param name fully qualified file name, e.g. java/lang/String
      * @param suffix file name ends with suff, e.g. .java
-     * @return class file for the java class
+     * @return class file for the Java class
      * @throws IOException if an I/O error occurs.
      */
     public ClassFile getClassFile(final String name, final String suffix) throws IOException {
@@ -674,32 +667,41 @@ public class ClassPath implements Closeable {
     }
 
     /**
+     * Gets an InputStream.
+     * <p>
+     * The caller is responsible for closing the InputStream.
+     * </p>
      * @param name fully qualified class name, e.g. java.lang.String
      * @return input stream for class
      * @throws IOException if an I/O error occurs.
      */
     public InputStream getInputStream(final String name) throws IOException {
-        return getInputStream(packageToFolder(name), ".class");
+        return getInputStream(Utility.packageToPath(name), JavaClass.EXTENSION);
     }
 
     /**
-     * Return stream for class or resource on CLASSPATH.
+     * Gets an InputStream for a class or resource on the classpath.
+     * <p>
+     * The caller is responsible for closing the InputStream.
+     * </p>
      *
-     * @param name fully qualified file name, e.g. java/lang/String
+     * @param name   fully qualified file name, e.g. java/lang/String
      * @param suffix file name ends with suff, e.g. .java
      * @return input stream for file on class path
      * @throws IOException if an I/O error occurs.
      */
     public InputStream getInputStream(final String name, final String suffix) throws IOException {
-        InputStream inputStream = null;
         try {
-            inputStream = getClass().getClassLoader().getResourceAsStream(name + suffix); // may return null
+            final java.lang.ClassLoader classLoader = getClass().getClassLoader();
+            @SuppressWarnings("resource") // closed by caller
+            final
+            InputStream inputStream = classLoader == null ? null : classLoader.getResourceAsStream(name + suffix);
+            if (inputStream != null) {
+                return inputStream;
+            }
         } catch (final Exception ignored) {
             // ignored
         }
-        if (inputStream != null) {
-            return inputStream;
-        }
         return getClassFile(name, suffix).getInputStream();
     }
 
@@ -776,10 +778,7 @@ public class ClassPath implements Closeable {
 
     @Override
     public int hashCode() {
-        if (parent != null) {
-            return classPath.hashCode() + parent.hashCode();
-        }
-        return classPath.hashCode();
+        return classPathString.hashCode();
     }
 
     /**
@@ -788,8 +787,8 @@ public class ClassPath implements Closeable {
     @Override
     public String toString() {
         if (parent != null) {
-            return parent + File.pathSeparator + classPath;
+            return parent + File.pathSeparator + classPathString;
         }
-        return classPath;
+        return classPathString;
     }
 }
